@@ -53,8 +53,8 @@ Given the data.csv above, fetch the JSON response.
 Note the output will be a JSONL file - with a minified JSON response per line, not a CSV file.
 
 Now, if we want to generate a CSV file with a parsed response - getting only the "form" property,
-we use the new-column and jql options. (See https://github.com/yamafaktory/jql#jql for more info 
-on how to use the jql JSON Query Language)
+we use the new-column and jql options. (See https://github.com/yamafaktory/jql#%EF%B8%8F-usage 
+for more info on how to use the jql JSON Query Language)
 
 $ qsv fetchpost URL zipcode,country --new-column form --jql '"form"' data.csv > data_with_response.csv
 
@@ -200,7 +200,7 @@ use simdutf8::basic::from_utf8;
 use url::Url;
 
 use crate::{
-    cmd::fetch::apply_jql,
+    cmd::fetch::process_jql,
     config::{Config, Delimiter},
     select::SelectColumns,
     util, CliError, CliResult,
@@ -277,7 +277,7 @@ impl RedisConfig {
                 .unwrap_or_else(|_| DEFAULT_FP_REDIS_TTL_SECS.to_string())
                 .parse()
                 .unwrap_or(DEFAULT_FP_REDIS_TTL_SECS),
-            ttl_refresh:   std::env::var("QSV_REDIS_TTL_REFRESH").is_ok(),
+            ttl_refresh:   util::get_envvar_flag("QSV_REDIS_TTL_REFRESH"),
         }
     }
 }
@@ -290,7 +290,6 @@ struct FetchResponse {
 }
 
 static REDISCONFIG: Lazy<RedisConfig> = Lazy::new(RedisConfig::load);
-static JQL_GROUPS: once_cell::sync::OnceCell<Vec<jql::Group>> = OnceCell::new();
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
@@ -394,15 +393,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     };
     info!("RATE LIMIT: {rate_limit}");
 
-    let user_agent = match args.flag_user_agent {
-        Some(ua) => match HeaderValue::from_str(ua.as_str()) {
-            Ok(_) => ua,
-            Err(e) => return fail_clierror!("Invalid user-agent value: {e}"),
-        },
-        None => util::DEFAULT_USER_AGENT.to_string(),
-    };
-    info!("USER-AGENT: {user_agent}");
-
     let http_headers: HeaderMap = {
         let mut map = HeaderMap::with_capacity(args.flag_http_header.len() + 1);
         for header in args.flag_http_header {
@@ -453,7 +443,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let client_timeout = time::Duration::from_secs(*TIMEOUT_FP_SECS.get().unwrap_or(&30));
     let client = Client::builder()
-        .user_agent(user_agent)
+        .user_agent(util::set_user_agent(args.flag_user_agent)?)
         .default_headers(http_headers)
         .cookie_store(args.flag_cookies)
         .brotli(true)
@@ -471,7 +461,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     // prep progress bars
     let show_progress =
-        (args.flag_progressbar || std::env::var("QSV_PROGRESSBAR").is_ok()) && !rconfig.is_stdin();
+        (args.flag_progressbar || util::get_envvar_flag("QSV_PROGRESSBAR")) && !rconfig.is_stdin();
 
     // create multi_progress to stderr with a maximum refresh of 5 per second
     let multi_progress = MultiProgress::with_draw_target(ProgressDrawTarget::stderr_with_hz(5));
@@ -544,13 +534,16 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         // url - URL used, form - form data sent, status - HTTP status code,
         // cache_hit - cache hit flag, retries - retry attempts,
         // elapsed - elapsed time (milliseconds) & response.
-        report_headers.push_field(format!("{rptcol_prefix}url").as_bytes());
-        report_headers.push_field(format!("{rptcol_prefix}form").as_bytes());
-        report_headers.push_field(format!("{rptcol_prefix}status").as_bytes());
-        report_headers.push_field(format!("{rptcol_prefix}cache_hit").as_bytes());
-        report_headers.push_field(format!("{rptcol_prefix}retries").as_bytes());
-        report_headers.push_field(format!("{rptcol_prefix}elapsed_ms").as_bytes());
-        report_headers.push_field(format!("{rptcol_prefix}response").as_bytes());
+        let report_header_fields = vec![
+            format!("{rptcol_prefix}url"),
+            format!("{rptcol_prefix}status"),
+            format!("{rptcol_prefix}cache_hit"),
+            format!("{rptcol_prefix}retries"),
+            format!("{rptcol_prefix}elapsed_ms"),
+            format!("{rptcol_prefix}response"),
+        ];
+        report_headers = csv::ByteRecord::from(report_header_fields);
+
         report_wtr.write_byte_record(&report_headers)?;
     }
 
@@ -993,11 +986,7 @@ fn get_response(
                 error_flag = false;
                 // apply JQL selector if provided
                 if let Some(selectors) = flag_jql {
-                    // instead of repeatedly parsing the jql selector,
-                    // we compile it only once and cache it for performance using once_cell
-                    let jql_groups =
-                        JQL_GROUPS.get_or_init(|| jql::selectors_parser(selectors).unwrap());
-                    match apply_jql(&api_value, jql_groups) {
+                    match process_jql(&api_value, selectors) {
                         Ok(s) => {
                             final_value = s;
                         }
