@@ -12,6 +12,9 @@ to adjust).
 If you want more robust, guaranteed schemata, use the "schema" or "stats" commands
 instead as they scan the entire file.
 
+On Linux, the command will also return the detected mime type. It uses the "libmagic"
+library to do this. See https://github.com/robo9k/rust-magic for more info.
+
 For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_sniff.rs.
 
 Usage:
@@ -47,6 +50,11 @@ sniff options:
                              Valid only when input is a URL.
     --timeout <secs>         Timeout for URL requests in seconds.
                              [default: 30]
+    --user-agent <agent>     Specify a custom user agent to use when sniffing a CSV on a URL.
+                             Try to follow the syntax here -
+                             https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/User-Agent
+    --stats-types            Use the same data type names as `stats`.
+                             (Unsigned, Signed => Integer, Text => String, everything else the same)
 
 Common options:
     -h, --help               Display this message
@@ -58,7 +66,13 @@ Common options:
     -p, --progressbar        Show progress bars. Only valid for URL input.
 "#;
 
-use std::{cmp::min, fmt, fs, io::Write, path::Path, time::Duration};
+use std::{
+    cmp::min,
+    fmt, fs,
+    io::Write,
+    path::PathBuf,
+    time::{Duration, SystemTime},
+};
 
 use bytes::Bytes;
 use futures::executor::block_on;
@@ -75,7 +89,7 @@ use url::Url;
 
 use crate::{
     config::{Config, Delimiter},
-    util, CliError, CliResult,
+    util, CliResult,
 };
 
 #[derive(Deserialize)]
@@ -89,18 +103,23 @@ struct Args {
     flag_delimiter:      Option<Delimiter>,
     flag_progressbar:    bool,
     flag_timeout:        u16,
+    flag_user_agent:     Option<String>,
+    flag_stats_types:    bool,
 }
 
 #[derive(Serialize, Deserialize, Default, Debug)]
 struct SniffStruct {
     path:            String,
     sniff_timestamp: String,
+    last_modified:   String,
     delimiter_char:  char,
     header_row:      bool,
     preamble_rows:   usize,
     quote_char:      String,
     flexible:        bool,
     is_utf8:         bool,
+    #[cfg(target_os = "linux")]
+    detected_mime:   String,
     retrieved_size:  usize,
     file_size:       usize,
     sampled_records: usize,
@@ -108,6 +127,7 @@ struct SniffStruct {
     num_records:     usize,
     avg_record_len:  usize,
     num_fields:      usize,
+    stats_types:     bool,
     fields:          Vec<String>,
     types:           Vec<String>,
 }
@@ -132,6 +152,7 @@ impl fmt::Display for SniffStruct {
             }
         )?;
         writeln!(f, "Sniff Timestamp: {}", self.sniff_timestamp)?;
+        writeln!(f, "Last Modified: {}", self.last_modified)?;
         writeln!(
             f,
             "Delimiter: {}",
@@ -150,6 +171,8 @@ impl fmt::Display for SniffStruct {
         writeln!(f, "Quote Char: {}", self.quote_char)?;
         writeln!(f, "Flexible: {}", self.flexible)?;
         writeln!(f, "Is UTF8: {}", self.is_utf8)?;
+        #[cfg(target_os = "linux")]
+        writeln!(f, "Detected Mime Type: {}", self.detected_mime)?;
         writeln!(
             f,
             "Retrieved Size (bytes): {}",
@@ -177,16 +200,25 @@ impl fmt::Display for SniffStruct {
             self.avg_record_len.separate_with_commas()
         )?;
         writeln!(f, "Num Fields: {}", self.num_fields.separate_with_commas())?;
+        writeln!(f, "Stats Types: {}", self.stats_types)?;
         writeln!(f, "Fields:")?;
 
         let mut tabwtr = TabWriter::new(vec![]);
 
         for (i, ty) in self.types.iter().enumerate() {
+            let data_type = if self.stats_types {
+                match ty.as_str() {
+                    "Unsigned" | "Signed" => "Integer",
+                    "Text" => "String",
+                    _ => ty,
+                }
+            } else {
+                ty
+            };
+
             writeln!(
                 &mut tabwtr,
-                "\t{}:\t{}\t{}",
-                i,
-                ty,
+                "\t{i}:\t{data_type}\t{}",
                 self.fields.get(i).unwrap_or(&String::new())
             )
             .unwrap_or_default();
@@ -207,6 +239,7 @@ struct SniffFileStruct {
     tempfile_flag:      bool,
     retrieved_size:     usize,
     file_size:          usize,
+    last_modified:      String,
     downloaded_records: usize,
 }
 
@@ -247,7 +280,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
 
                 // setup the reqwest client
                 let client = match Client::builder()
-                    .user_agent(util::DEFAULT_USER_AGENT)
+                    .user_agent(util::set_user_agent(args.flag_user_agent.clone())?)
                     .brotli(true)
                     .gzip(true)
                     .deflate(true)
@@ -270,6 +303,19 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     .await
                     .or(Err(format!("Failed to GET from '{url}'")))?;
 
+                let last_modified = match res.headers().get("Last-Modified") {
+                    Some(lm) => match lm.to_str() {
+                        Ok(s) => {
+                            // convert Last-Modified RFC2822 to RFC3339 format
+                            let dt = chrono::DateTime::parse_from_rfc2822(s).unwrap();
+                            dt.to_rfc3339_opts(chrono::SecondsFormat::Secs, false)
+                        }
+                        // server did not return Last-Modified header
+                        Err(_) => String::from("Unknown"),
+                    },
+                    None => String::new(),
+                };
+
                 let total_size = match res.content_length() {
                     Some(l) => l as usize,
                     None => {
@@ -340,6 +386,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                 // download chunks until we have the desired sample size
                 while let Some(item) = stream.next().await {
                     chunk = item.or(Err("Error while downloading file".to_string()))?;
+
                     file.write_all(&chunk)
                         .map_err(|_| "Error while writing to file".to_string())?;
                     let chunk_len = chunk.len();
@@ -393,13 +440,14 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                 if snappy_flag {
                     // we downloaded a snappy compressed file, we need to decompress it
                     // before we can sniff it
-                    wtr_file_path = decompress_snappy_file(file.path().to_str().unwrap(), tmpdir)?;
+                    wtr_file_path =
+                        util::decompress_snappy_file(&file.path().to_path_buf(), tmpdir)?;
                 } else {
-                    // we downloaded a non-snappy file, rewrite it so we only have the exact
+                    // we downloaded a non-snappy file. Rewrite it so we only have the exact
                     // sample size and truncate potentially incomplete lines. We streamed the
                     // download and the downloaded file may be more than the sample size, and the
                     // final line may be incomplete.
-                    wtr_file_path = path.to_str().unwrap().to_string();
+                    wtr_file_path = path.display().to_string();
                     let mut wtr = Config::new(&Some(wtr_file_path.clone()))
                         .no_headers(false)
                         .flexible(true)
@@ -448,30 +496,86 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     } else {
                         total_size
                     },
+                    last_modified,
                     downloaded_records,
                 })
             }
-            // its a file, passthrough the path along with its size
+            // its a file. If its a snappy file, decompress it first
+            // aftwerwards, check if its one of the supported file types
+            // finally, check if its a utf8 file
             path => {
                 let mut path = path;
 
-                if path.to_lowercase().ends_with(".sz") {
-                    path = decompress_snappy_file(&path, tmpdir)?;
+                let mut pathbuf = PathBuf::from(path.clone());
+                let file_ext = pathbuf.extension();
+                match file_ext {
+                    Some(ext) => {
+                        let mut lower_ext =
+                            ext.to_str().unwrap().to_lowercase().as_str().to_owned();
+                        if lower_ext == "sz" {
+                            path = util::decompress_snappy_file(&pathbuf, tmpdir)?;
+                            pathbuf = PathBuf::from(path.clone());
+                            lower_ext = pathbuf
+                                .extension()
+                                .unwrap()
+                                .to_os_string()
+                                .into_string()
+                                .unwrap();
+                            log::info!("Decompressed {lower_ext} file to {path}");
+                        }
+                        // on linux, we don't need to check the extension
+                        // because we use magic to get the file type
+                        #[cfg(not(target_os = "linux"))]
+                        match lower_ext.as_str() {
+                            "csv" | "tsv" | "txt" | "tab" => {}
+                            ext if ext.ends_with("_decompressed") => {}
+                            _ => {
+                                return fail_clierror!(
+                                    "File extension '{lower_ext}' is not supported",
+                                    // ext = ext.to_str().unwrap()
+                                );
+                            }
+                        }
+                    }
+                    None => {
+                        // on linux, we log a warning and continue if no
+                        // extension is found. On other platforms, we fail
+                        #[cfg(not(target_os = "linux"))]
+                        return fail_clierror!("File extension not found");
+                        #[cfg(target_os = "linux")]
+                        log::warn!("File extension not found");
+                    }
                 }
 
                 let metadata = fs::metadata(&path)
                     .map_err(|_| format!("Cannot get metadata for file '{path}'"))?;
 
-                let fsize = metadata.len() as usize;
+                let file_size = metadata.len() as usize;
+                let last_modified = match metadata.modified() {
+                    Ok(time) => {
+                        let timestamp = time
+                            .duration_since(SystemTime::UNIX_EPOCH)
+                            .unwrap()
+                            .as_secs();
+                        let naive = chrono::NaiveDateTime::from_timestamp_opt(timestamp as i64, 0)
+                            .unwrap_or_default();
+                        let datetime =
+                            chrono::DateTime::<chrono::Utc>::from_utc(naive, chrono::Utc);
+                        // format the datetime to RFC3339
+                        format!("{datetime}", datetime = datetime.format("%+"))
+                    }
+                    Err(_) => "N/A".to_string(),
+                };
 
                 let canonical_path = fs::canonicalize(&path)?.to_str().unwrap().to_string();
 
                 Ok(SniffFileStruct {
-                    display_path:       canonical_path,
-                    file_to_sniff:      path,
-                    tempfile_flag:      false,
-                    retrieved_size:     fsize,
-                    file_size:          fsize,
+                    display_path: canonical_path,
+                    file_to_sniff: path,
+                    tempfile_flag: false,
+                    retrieved_size: file_size,
+                    file_size,
+                    last_modified,
                     downloaded_records: 0,
                 })
             }
@@ -487,40 +591,46 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
             .keep()
             .or(Err("Cannot keep temporary file".to_string()))?;
 
+        if !util::isutf8_file(&path)? {
+            return fail_clierror!("File is not UTF8-encoded nor a text/CSV file");
+        }
+
         let metadata = file
             .metadata()
             .map_err(|_| "Cannot get metadata for stdin file".to_string())?;
 
-        let fsize = metadata.len() as usize;
+        let file_size = metadata.len() as usize;
+        let last_modified = match metadata.modified() {
+            Ok(time) => {
+                let timestamp = time
+                    .duration_since(SystemTime::UNIX_EPOCH)
+                    .unwrap()
+                    .as_secs();
+                let naive = chrono::NaiveDateTime::from_timestamp_opt(timestamp as i64, 0)
+                    .unwrap_or_default();
+                let datetime = chrono::DateTime::<chrono::Utc>::from_utc(naive, chrono::Utc);
+                // format the datetime to RFC3339
+                format!("{datetime}", datetime = datetime.format("%+"))
+            }
+            Err(_) => "N/A".to_string(),
+        };
         let path_string = path
             .into_os_string()
             .into_string()
             .unwrap_or_else(|_| "???".to_string());
 
         Ok(SniffFileStruct {
-            display_path:       "stdin".to_string(),
-            file_to_sniff:      path_string,
-            tempfile_flag:      true,
-            retrieved_size:     fsize,
-            file_size:          fsize,
+            display_path: "stdin".to_string(),
+            file_to_sniff: path_string,
+            tempfile_flag: true,
+            retrieved_size: file_size,
+            file_size,
+            last_modified,
             downloaded_records: 0,
         })
     }
 }
 
-fn decompress_snappy_file(path: &str, tmpdir: &tempfile::TempDir) -> Result<String, CliError> {
-    let mut snappy_file = std::fs::File::open(path)?;
-    let mut snappy_reader = snap::read::FrameDecoder::new(&mut snappy_file);
-    let file_stem = Path::new(path).file_stem().unwrap().to_str().unwrap();
-    let decompressed_filepath = tmpdir
-        .path()
-        .join(format!("qsv__{file_stem}__qsv_temp_decompressed"));
-    let mut decompressed_file = std::fs::File::create(decompressed_filepath.clone())?;
-    std::io::copy(&mut snappy_reader, &mut decompressed_file)?;
-    decompressed_file.flush()?;
-    Ok(format!("{}", decompressed_filepath.display()))
-}
-
 fn cleanup_tempfile(
     tempfile_flag: bool,
     tempfile: String,
@@ -556,10 +666,34 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
     let sfile_info = block_on(future)?;
     let tempfile_to_delete = sfile_info.file_to_sniff.clone();
 
+    // on linux, check what kind of file we have
+    // if its NOT a CSV or a text file, we fail, showing the detected mime type
+    #[cfg(target_os = "linux")]
+    let file_type = util::get_filetype(&sfile_info.file_to_sniff)?;
+    #[cfg(target_os = "linux")]
+    if file_type != "application/csv" && !file_type.starts_with("text/") {
+        cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
+        if args.flag_json || args.flag_pretty_json {
+            let json_result = json!({
+                "errors": [{
+                    "title": "sniff error",
+                    "detail": format!("File is not a CSV file. Detected mime type: {file_type}"),
+                }]
+            });
+            if args.flag_pretty_json {
+                return fail_clierror!("{}", serde_json::to_string_pretty(&json_result).unwrap());
+            }
+            return fail_clierror!("{json_result}");
+        }
+        return fail_clierror!("File is not a CSV file. Detected mime type: {file_type}");
+    }
+
     let conf = Config::new(&Some(sfile_info.file_to_sniff.clone()))
         .flexible(true)
         .delimiter(args.flag_delimiter);
-    let n_rows = if sfile_info.downloaded_records == 0 {
+    let n_rows = if sfile_info.downloaded_records == 0
+        || sfile_info.retrieved_size <= sfile_info.file_size
+    {
         //if we have the whole file and not just a sample, we can count the number of rows
         match util::count_rows(&conf) {
             Ok(n) => n as usize,
@@ -690,6 +824,7 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
             processed_results = SniffStruct {
                 path: sfile_info.display_path,
                 sniff_timestamp: sniffed_ts,
+                last_modified: sfile_info.last_modified,
                 delimiter_char: metadata.dialect.delimiter as char,
                 header_row: metadata.dialect.header.has_header_row,
                 preamble_rows: metadata.dialect.header.num_preamble_rows,
@@ -699,6 +834,8 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
                 },
                 flexible: metadata.dialect.flexible,
                 is_utf8: metadata.dialect.is_utf8,
+                #[cfg(target_os = "linux")]
+                detected_mime: file_type,
                 retrieved_size: sfile_info.retrieved_size,
                 file_size: sfile_info.file_size, // sfile_info.file_size,
                 sampled_records: if sampled_records > num_records {
@@ -712,10 +849,18 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
                 num_fields: metadata.num_fields,
                 fields: sniffedfields,
                 types: sniffedtypes,
+                stats_types: args.flag_stats_types,
             };
         }
         Err(e) => {
-            sniffing_error = Some(e.to_string());
+            #[cfg(target_os = "linux")]
+            {
+                sniffing_error = Some(format!("{e}. Detected mime type: {file_type}"));
+            }
+            #[cfg(not(target_os = "linux"))]
+            {
+                sniffing_error = Some(format!("{e}"));
+            }
         }
     }
 
