@@ -136,7 +136,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     let show_progress =
-        (args.flag_progressbar || std::env::var("QSV_PROGRESSBAR").is_ok()) && !rconfig.is_stdin();
+        (args.flag_progressbar || util::get_envvar_flag("QSV_PROGRESSBAR")) && !rconfig.is_stdin();
 
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     if show_progress {
@@ -911,7 +911,9 @@ fn load_json(uri: &str) -> Result<String, String> {
                 std::time::Duration::from_secs(TIMEOUT_SECS.load(Ordering::Relaxed) as u64);
 
             let client = match Client::builder()
-                .user_agent(util::DEFAULT_USER_AGENT)
+                // safety: we're using a validated QSV_USER_AGENT or if it's not set,
+                // the default user agent
+                .user_agent(util::set_user_agent(None).unwrap())
                 .brotli(true)
                 .gzip(true)
                 .deflate(true)
