@@ -2,6 +2,7 @@
 use std::borrow::Cow;
 use std::{
     env, fs,
+    io::{BufReader, Read, Write},
     path::{Path, PathBuf},
     str,
 };
@@ -16,6 +17,7 @@ use serde::de::{Deserialize, Deserializer, Error};
 use sysinfo::{System, SystemExt};
 
 use crate::{
+    config,
     config::{Config, Delimiter},
     CliError, CliResult,
 };
@@ -229,6 +231,7 @@ pub fn show_env_vars() {
     let mut env_var_set = false;
     for (n, v) in env::vars_os() {
         let env_var = n.into_string().unwrap();
+        #[cfg(feature = "mimalloc")]
         if env_var.starts_with("QSV_")
             || env_var.starts_with("MIMALLOC_")
             || OTHER_ENV_VARS.contains(&env_var.to_lowercase().as_str())
@@ -236,6 +239,21 @@ pub fn show_env_vars() {
             env_var_set = true;
             woutinfo!("{env_var}: {v:?}");
         }
+        #[cfg(feature = "jemallocator")]
+        if env_var.starts_with("QSV_")
+            || env_var.starts_with("JEMALLOC_")
+            || env_var.starts_with("MALLOC_CONF")
+            || OTHER_ENV_VARS.contains(&env_var.to_lowercase().as_str())
+        {
+            env_var_set = true;
+            woutinfo!("{env_var}: {v:?}");
+        }
+        #[cfg(not(any(feature = "mimalloc", feature = "jemallocator")))]
+        if env_var.starts_with("QSV_") || OTHER_ENV_VARS.contains(&env_var.to_lowercase().as_str())
+        {
+            env_var_set = true;
+            woutinfo!("{env_var}: {v:?}");
+        }
     }
     if !env_var_set {
         woutinfo!("No qsv-relevant environment variables set.");
@@ -1214,3 +1232,52 @@ pub fn get_envvar_flag(key: &str) -> bool {
         false
     }
 }
+
+pub fn decompress_snappy_file(
+    path: &PathBuf,
+    tmpdir: &tempfile::TempDir,
+) -> Result<String, CliError> {
+    let mut snappy_file = std::fs::File::open(path.clone())?;
+    let mut snappy_reader = snap::read::FrameDecoder::new(&mut snappy_file);
+    let file_stem = Path::new(&path).file_stem().unwrap().to_str().unwrap();
+    let decompressed_filepath = tmpdir
+        .path()
+        .join(format!("qsv__{file_stem}__qsv_temp_decompressed"));
+    let mut decompressed_file = std::fs::File::create(decompressed_filepath.clone())?;
+    std::io::copy(&mut snappy_reader, &mut decompressed_file)?;
+    decompressed_file.flush()?;
+    Ok(format!("{}", decompressed_filepath.display()))
+}
+
+/// load the first BUFFER*4 (128k) bytes of the file and check if it is utf8
+pub fn isutf8_file(path: &Path) -> Result<bool, CliError> {
+    let metadata = std::fs::metadata(path)?;
+    let buffer_len = config::DEFAULT_RDR_BUFFER_CAPACITY * 4;
+    let file_size = metadata.len() as usize;
+    let bytes_to_read: usize = if file_size < buffer_len {
+        file_size
+    } else {
+        buffer_len
+    };
+
+    let file = std::fs::File::open(path)?;
+    let mut reader = BufReader::new(file);
+    let mut buffer = Vec::with_capacity(bytes_to_read);
+    reader.read_to_end(&mut buffer)?;
+
+    Ok(simdutf8::basic::from_utf8(&buffer).is_ok())
+}
+
+/// find out what kind of file we have using magic
+#[cfg(target_os = "linux")]
+pub fn get_filetype(path: &str) -> Result<String, CliError> {
+    // We just want the mime type
+    let cookie = magic::Cookie::open(magic::CookieFlags::MIME_TYPE)?;
+
+    // Load libmagic's default database
+    cookie.load::<&str>(&[])?;
+
+    let mime = cookie.file(path)?;
+
+    Ok(mime)
+}
