@@ -18,27 +18,22 @@ fn data() -> Vec<Vec<String>> {
 #[test]
 fn sniff() {
     let wrk = Workdir::new("sniff");
-    wrk.create_with_delim("in.file", data(), b',');
+    wrk.create_with_delim("in.CSV", data(), b',');
 
     let mut cmd = wrk.command("sniff");
-    cmd.arg("in.file");
+    cmd.arg("in.CSV");
 
     let got: String = wrk.stdout(&mut cmd);
 
     let expected_end = dos2unix(
-        r#"Delimiter: ,
-Header Row: true
-Preamble Rows: 0
-Quote Char: none
-Flexible: false
-Is UTF8: true
-Retrieved Size (bytes): 27
+        r#"Retrieved Size (bytes): 27
 File Size (bytes): 27
 Sampled Records: 2
 Estimated: false
 Num Records: 2
 Avg Record Len (bytes): 6
 Num Fields: 3
+Stats Types: false
 Fields:
     0:  Text      h1
     1:  Unsigned  h2
@@ -49,19 +44,98 @@ Fields:
 }
 
 #[test]
-fn sniff_url() {
-    let wrk = Workdir::new("sniff_url");
+fn sniff_stats_types() {
+    let wrk = Workdir::new("sniff_stats_types");
+    wrk.create_with_delim("in.CSV", data(), b',');
 
     let mut cmd = wrk.command("sniff");
-    cmd.arg("https://github.com/jqnatividad/qsv/raw/master/resources/test/boston311-100.csv");
+    cmd.arg("--stats-types").arg("in.CSV");
+
+    let got: String = wrk.stdout(&mut cmd);
+
+    let expected_end = dos2unix(
+        r#"Retrieved Size (bytes): 27
+File Size (bytes): 27
+Sampled Records: 2
+Estimated: false
+Num Records: 2
+Avg Record Len (bytes): 6
+Num Fields: 3
+Stats Types: true
+Fields:
+    0:  String   h1
+    1:  Integer  h2
+    2:  String   h3"#,
+    );
+
+    assert!(dos2unix(&got).trim_end().ends_with(expected_end.trim_end()));
+}
+
+#[test]
+fn sniff_url_notcsv() {
+    let wrk = Workdir::new("sniff_url_notcsv");
+
+    let mut cmd = wrk.command("sniff");
+    cmd.arg("https://github.com/jqnatividad/qsv/raw/master/resources/test/excel-xls.xls");
+
+    #[cfg(target_os = "linux")]
+    {
+        let got_error = wrk.output_stderr(&mut cmd);
+
+        let expected = "File is not a CSV file. Detected mime type: application/octet-stream";
+        assert_eq!(
+            dos2unix(&got_error).trim_end(),
+            dos2unix(expected).trim_end()
+        );
+    }
+    #[cfg(not(target_os = "linux"))]
+    {
+        wrk.assert_err(&mut cmd);
+    }
+}
+
+#[test]
+fn sniff_notcsv() {
+    let wrk = Workdir::new("sniff_notcsv");
+
+    let test_file = wrk.load_test_file("excel-xls.xls");
+
+    let mut cmd = wrk.command("sniff");
+    cmd.arg(test_file);
+
+    let got_error = wrk.output_stderr(&mut cmd);
+
+    let expected;
+    #[cfg(target_os = "linux")]
+    {
+        expected = "File is not a CSV file. Detected mime type: application/vnd.ms-excel";
+    }
+    #[cfg(not(target_os = "linux"))]
+    {
+        expected = "File extension 'xls' is not supported";
+    }
+
+    assert_eq!(
+        dos2unix(&got_error).trim_end(),
+        dos2unix(expected).trim_end()
+    );
+}
+
+#[test]
+fn sniff_url_snappy() {
+    let wrk = Workdir::new("sniff_url_snappy");
+
+    let mut cmd = wrk.command("sniff");
+    cmd.arg("https://github.com/jqnatividad/qsv/raw/master/resources/test/boston311-100.csv.sz");
 
     let got: String = wrk.stdout(&mut cmd);
 
     let expected_end = r#"Sampled Records: 100
-Estimated: true
-Num Records: 107
+Estimated: false
+Num Records: 100
 Avg Record Len (bytes): 444
 Num Fields: 29
+Stats Types: false
 Fields:
     0:   Unsigned  case_enquiry_id
     1:   DateTime  open_dt
@@ -77,7 +151,7 @@ Fields:
     11:  Text      queue
     12:  Text      department
     13:  Text      submittedphoto
-    14:  Boolean   closedphoto
+    14:  NULL      closedphoto
     15:  Text      location
     16:  Unsigned  fire_district
     17:  Text      pwd_district
@@ -97,11 +171,13 @@ Fields:
 }
 
 #[test]
-fn sniff_url_snappy() {
-    let wrk = Workdir::new("sniff_url_snappy");
+fn sniff_file_snappy() {
+    let wrk = Workdir::new("sniff_file_snappy");
+
+    let test_file = wrk.load_test_file("boston311-100.csv.sz");
 
     let mut cmd = wrk.command("sniff");
-    cmd.arg("https://github.com/jqnatividad/qsv/raw/master/resources/test/boston311-100.csv.sz");
+    cmd.arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
 
@@ -110,6 +186,7 @@ Estimated: false
 Num Records: 100
 Avg Record Len (bytes): 444
 Num Fields: 29
+Stats Types: false
 Fields:
     0:   Unsigned  case_enquiry_id
     1:   DateTime  open_dt
@@ -125,7 +202,7 @@ Fields:
     11:  Text      queue
     12:  Text      department
     13:  Text      submittedphoto
-    14:  Boolean   closedphoto
+    14:  NULL      closedphoto
     15:  Text      location
     16:  Unsigned  fire_district
     17:  Text      pwd_district
@@ -147,14 +224,40 @@ Fields:
 #[test]
 fn sniff_tab() {
     let wrk = Workdir::new("sniff_tab");
-    wrk.create_with_delim("in.file", data(), b'\t');
+    wrk.create_with_delim("in.TAB", data(), b'\t');
 
     let mut cmd = wrk.command("sniff");
-    cmd.arg("in.file");
+    cmd.arg("in.TAB");
 
     let got: String = wrk.stdout(&mut cmd);
 
-    let expected_end = r#"Delimiter: tab
+    let expected_end;
+
+    #[cfg(target_os = "linux")]
+    {
+        expected_end = r#"Delimiter: tab
+Header Row: true
+Preamble Rows: 0
+Quote Char: none
+Flexible: false
+Is UTF8: true
+Detected Mime Type: text/plain
+Retrieved Size (bytes): 27
+File Size (bytes): 27
+Sampled Records: 2
+Estimated: false
+Num Records: 2
+Avg Record Len (bytes): 6
+Num Fields: 3
+Stats Types: false
+Fields:
+    0:  Text      h1
+    1:  Unsigned  h2
+    2:  Text      h3"#;
+    }
+    #[cfg(not(target_os = "linux"))]
+    {
+        expected_end = r#"Delimiter: tab
 Header Row: true
 Preamble Rows: 0
 Quote Char: none
@@ -167,10 +270,12 @@ Estimated: false
 Num Records: 2
 Avg Record Len (bytes): 6
 Num Fields: 3
+Stats Types: false
 Fields:
     0:  Text      h1
     1:  Unsigned  h2
     2:  Text      h3"#;
+    }
 
     assert!(dos2unix(&got).trim_end().ends_with(expected_end.trim_end()));
 }
@@ -223,7 +328,7 @@ fn sniff_json() {
     cmd.arg("--json").arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
-    let expected_end: &str = r#"ampled_records":3,"estimated":false,"num_records":3,"avg_record_len":10,"num_fields":4,"fields":["h1","h2","h3","h4"],"types":["Text","Unsigned","Text","Float"]}"#;
+    let expected_end: &str = r#"ampled_records":3,"estimated":false,"num_records":3,"avg_record_len":10,"num_fields":4,"stats_types":false,"fields":["h1","h2","h3","h4"],"types":["Text","Unsigned","Text","Float"]}"#;
 
     assert!(got.ends_with(expected_end));
 }
@@ -238,7 +343,7 @@ fn sniff_flexible_json() {
 
     let got: String = wrk.stdout(&mut cmd);
 
-    let expected_end = r#"sampled_records":5,"estimated":false,"num_records":5,"avg_record_len":8,"num_fields":4,"fields":["h1","h2","h3","h4"],"types":["Text","Unsigned","Text","Float"]}"#;
+    let expected_end = r#"sampled_records":5,"estimated":false,"num_records":5,"avg_record_len":8,"num_fields":4,"stats_types":false,"fields":["h1","h2","h3","h4"],"types":["Text","Unsigned","Text","Float"]}"#;
 
     assert!(got.ends_with(expected_end));
 }
@@ -259,6 +364,7 @@ fn sniff_pretty_json() {
   "num_records": 3,
   "avg_record_len": 10,
   "num_fields": 4,
+  "stats_types": false,
   "fields": [
     "h1",
     "h2",
@@ -295,6 +401,7 @@ fn sniff_sample() {
   "num_records": 15,
   "avg_record_len": 555,
   "num_fields": 32,
+  "stats_types": false,
   "fields": [
     "ExtractDate",
     "OrganisationURI",
@@ -342,15 +449,15 @@ fn sniff_sample() {
     "Text",
     "Text",
     "Text",
-    "Text",
-    "Text",
-    "Text",
-    "Text",
-    "Text",
-    "Text",
-    "Text",
     "Boolean",
     "Boolean",
+    "Boolean",
+    "Boolean",
+    "Boolean",
+    "Boolean",
+    "Boolean",
+    "NULL",
+    "NULL",
     "Text",
     "Text",
     "Text",
@@ -358,10 +465,10 @@ fn sniff_sample() {
     "Text",
     "Text",
     "Unsigned",
-    "Boolean",
+    "NULL",
     "Text",
-    "Boolean",
-    "Boolean"
+    "NULL",
+    "NULL"
   ]
 }"#;
 
@@ -383,6 +490,7 @@ Estimated: false
 Num Records: 100
 Avg Record Len (bytes): 444
 Num Fields: 29
+Stats Types: false
 Fields:
     0:   Unsigned  case_enquiry_id
     1:   DateTime  open_dt
@@ -398,7 +506,7 @@ Fields:
     11:  Text      queue
     12:  Text      department
     13:  Text      submittedphoto
-    14:  Boolean   closedphoto
+    14:  NULL      closedphoto
     15:  Text      location
     16:  Unsigned  fire_district
     17:  Text      pwd_district
@@ -430,3 +538,33 @@ fn sniff_flaky_delimiter_guess() {
     // guess the wrong delimiter if we don't explicitly set it.
     wrk.assert_success(&mut cmd);
 }
+
+#[test]
+fn sniff_consistent_results_issue_956() {
+    let wrk = Workdir::new("sniff_consistent_results_issue_956");
+
+    let test_file = wrk.load_test_file("spendover25kdownloadSep.csv");
+    let mut cmd = wrk.command("sniff");
+    cmd.arg(test_file);
+    wrk.assert_err(&mut cmd);
+
+    let test_file = wrk.load_test_file("311011.csv");
+    let mut cmd = wrk.command("sniff");
+    cmd.arg(test_file);
+    wrk.assert_err(&mut cmd);
+
+    let test_file = wrk.load_test_file("FCOServices_TransparencySpend_May2011.csv");
+    let mut cmd = wrk.command("sniff");
+    cmd.arg(test_file);
+    wrk.assert_err(&mut cmd);
+
+    let test_file = wrk.load_test_file("iwfg09_Phos_river_200911.csv");
+    let mut cmd = wrk.command("sniff");
+    cmd.arg(test_file);
+    wrk.assert_err(&mut cmd);
+
+    let test_file = wrk.load_test_file("Inpatients_MHA_Machine_readable_dataset_1011.csv");
+    let mut cmd = wrk.command("sniff");
+    cmd.arg(test_file);
+    wrk.assert_err(&mut cmd);
+}
